/****************************************************************************
 ****************************************************************************
 **
 **   File: create_graphs.c
 **
 **   Description: Creates graphs for the Xstation diagnostic monitor.
 **
 ****************************************************************************
 ****************************************************************************/

/************************************************************
*                                                           *
*  Revision history:                                        *
*                                                           *
*  02/09/91  Gary Goldman, IBM Austin   Initial Version     *
*                                                           *
*************************************************************/

#include "xmonitor.h"

/*****************************************************************************
 *                        Globals
 *****************************************************************************/
extern int help_index;

/***************************************************************************
 * Create graphs.
 ***************************************************************************/ 
void create_graphs()
{
	Widget h_bar, v_bar, button;
	Arg args [20];
	XmString label;
	int i, n, nsav;
 	static int h_cback[NUM_GRAPHS][2], v_cback[NUM_GRAPHS][2];

	FOR_ALL_GRAPHS(i) {
		n = 0;
		label = XmStringLtoRCreate(graph[i].title, XmSTRING_DEFAULT_CHARSET);
		XtSetArg(args[n], XmNdialogTitle, label); n++;
		XtSetArg(args[n], XmNdefaultPosition, FALSE); n++;
		XtSetArg(args[n], XmNx, graph[i].geom.position.x); n++;
		XtSetArg(args[n], XmNy, graph[i].geom.position.y); n++;

		graph[i].dialog = XmCreateBulletinBoardDialog(toplevel_widget, 
												"GRAPH_DIALOG0", args, n);

		graph[i].form = XmCreateForm(graph[i].dialog, "GRAPH_FORM0", NULL, 0);
		XtManageChild(graph[i].form);
		XtAddEventHandler(graph[i].form, ExposureMask, FALSE, expose_form, i);

		n = 0;
		XtSetArg(args[n], XmNbackground, graph[i].background); n++;
		XtSetArg(args[n], XmNtopAttachment, XmATTACH_FORM); n++;
		XtSetArg(args[n], XmNtopOffset, graph_prop.widget_offset); n++;
		XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
		XtSetArg(args[n], XmNleftOffset, graph_prop.left_offset); n++;
		XtSetArg(args[n], XmNwidth, 
				 graph[i].geom.inc_pixels.x*graph[i].view.size.x); n++;
		XtSetArg(args[n], XmNheight, 
				 graph[i].geom.inc_pixels.y*graph[i].view.size.y); n++;
		graph[i].d_area = XmCreateDrawingArea(graph[i].form, "GRAPH_DA0", 
											  args, n);
		XtManageChild(graph[i].d_area);
		XtAddCallback(graph[i].d_area, XmNexposeCallback, expose_graph, i);
		n = 0;
		XtSetArg(args[n], XmNorientation, XmHORIZONTAL); n++;
		XtSetArg(args[n], XmNincrement, 1); n++;
		XtSetArg(args[n], XmNvalue, graph[i].geom.max_inc.x-
					graph[i].view.size.x); n++;
		XtSetArg(args[n], XmNmaximum, 
				 graph[i].geom.max_inc.x-graph[i].view.size.x+1); n++;
		XtSetArg(args[n], XmNpageIncrement, 4); n++;
		XtSetArg(args[n], XmNsliderSize, 1); n++;
		XtSetArg(args[n], XmNleftAttachment, XmATTACH_OPPOSITE_WIDGET); n++;
		XtSetArg(args[n], XmNleftWidget, graph[i].d_area); n++;
		XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
		XtSetArg(args[n], XmNtopWidget, graph[i].d_area); n++;
		XtSetArg(args[n], XmNtopOffset, graph_prop.widget_offset); n++;
		XtSetArg(args[n], XmNrightAttachment, XmATTACH_OPPOSITE_WIDGET); n++;
		XtSetArg(args[n], XmNrightWidget, graph[i].d_area); n++;
		h_bar = XmCreateScrollBar(graph[i].form, "GRAPH_HBAR0", args, n);
		XtManageChild(h_bar);
		h_cback[i][0] = i;
		h_cback[i][1] = HORIZONTAL;
		XtAddCallback(h_bar, XmNvalueChangedCallback, scroll_graph, h_cback[i]);

		n = 0;
		XtSetArg(args[n], XmNprocessingDirection, XmMAX_ON_TOP); n++;
		XtSetArg(args[n], XmNincrement, 1); n++;
		XtSetArg(args[n], XmNmaximum, 
				 graph[i].geom.max_inc.y-graph[i].view.size.y+1); n++;
		XtSetArg(args[n], XmNpageIncrement, 4); n++;
		XtSetArg(args[n], XmNsliderSize, 1); n++;
		XtSetArg(args[n], XmNtopAttachment, XmATTACH_OPPOSITE_WIDGET); n++;
		XtSetArg(args[n], XmNtopWidget, graph[i].d_area); n++;
		XtSetArg(args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
		XtSetArg(args[n], XmNleftWidget, graph[i].d_area); n++;
		XtSetArg(args[n], XmNleftOffset, graph_prop.left_offset); n++;
		XtSetArg(args[n], XmNbottomAttachment, XmATTACH_OPPOSITE_WIDGET); n++;
		XtSetArg(args[n], XmNbottomWidget, graph[i].d_area); n++;
		v_bar = XmCreateScrollBar(graph[i].form, "GRAPH_VBAR0", args, n);
		XtManageChild(v_bar);
		v_cback[i][0] = i;
		v_cback[i][1] = VERTICAL;
		XtAddCallback(v_bar, XmNvalueChangedCallback, scroll_graph, v_cback[i]);

    	n = 0;
    	label = XmStringLtoRCreate("Close", XmSTRING_DEFAULT_CHARSET);
    	XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    	XtSetArg(args[n], XmNtopWidget, h_bar); n++;
    	XtSetArg(args[n], XmNtopOffset, graph_prop.widget_offset); n++;
		nsav = n;
   		XtSetArg(args[n], XmNlabelString, label); n++;
    	XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
    	button = XmCreatePushButtonGadget(graph[i].form, "GRAPH_CLOSE", 
										  args, n);
    	XtManageChild(button);
    	XtAddCallback(button, XmNactivateCallback, remove_graph, i);

		n = nsav;
		label = XmStringLtoRCreate("Help", XmSTRING_DEFAULT_CHARSET);
		XtSetArg(args[n], XmNlabelString, label); n++;
		XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
		button = XmCreatePushButtonGadget(graph[i].form, "GRAPH_HELP", args, n);
		XtManageChild(button);
		XtAddCallback(button, XmNactivateCallback, activate_help, help_index);
		strcpy(help[help_index].title, graph[i].title);
		strcat(help[help_index++].title, " Help");

		create_graph_labels(i); /* annotate graph y axis */

		XtManageChild(graph[i].dialog);
	}
	XmStringFree(label);
}

/***************************************************************************
 * Create graph y axis labels.
 ***************************************************************************/
static void create_graph_labels(gr_num)
	int gr_num;
{
	Arg args[20];
	XmString label;
	int i, n, nsav[2], side;
	Dimension offset;

	label = XmStringLtoRCreate("1", XmSTRING_DEFAULT_CHARSET);
	offset = (XmStringHeight(tiny_font,label))/2;

	n = 0;
	XtSetArg(args[n], XmNfontList, tiny_font); n++;
	XtSetArg(args[n], XmNborderWidth, 0); n++;
	XtSetArg(args[n], XmNmarginWidth, 0); n++;
	XtSetArg(args[n], XmNmarginHeight, 0); n++;
	XtSetArg(args[n], XmNbottomAttachment, XmATTACH_OPPOSITE_WIDGET); n++;
	nsav[0] = n;
	for (side = 0; side < 2; side++) {
		n = nsav[0];
		if (!side) {
			XtSetArg(args[n], XmNrightAttachment, XmATTACH_WIDGET); n++;
			XtSetArg(args[n], XmNrightWidget, graph[gr_num].d_area); n++;
			XtSetArg(args[n], XmNrightOffset, graph_prop.tick_size+5); n++;
		}
		else {
			XtSetArg(args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
			XtSetArg(args[n], XmNleftWidget, graph[gr_num].d_area); n++;
			XtSetArg(args[n], XmNleftOffset, graph_prop.tick_size+5); n++;
		}
		nsav[1] = n;
		for (i = 0; i < graph[gr_num].labels[side].num; i++) {
			n = nsav[1];
			if (!i) {
				XtSetArg(args[n], XmNbottomWidget, graph[gr_num].d_area); n++;
				XtSetArg(args[n], XmNbottomOffset, -offset); n++;
			}
			else {
				XtSetArg(args[n], XmNbottomWidget, 
						 graph[gr_num].labels[side].widget[0]); n++;
				XtSetArg(args[n], XmNbottomOffset, 
				  		 i*graph[gr_num].geom.inc_pixels.y*
				  		 graph[gr_num].labels[side].inc); n++;
			}
   			graph[gr_num].labels[side].widget[i] = 
			   XmCreateLabelGadget(graph[gr_num].form, "GRAPH_LABELS", args, n);
   			XtManageChild(graph[gr_num].labels[side].widget[i]);
		}
	}
	set_graph_labels(gr_num);
	XmStringFree(label);
}
