/****************************************************************************
 ****************************************************************************
 **
 **   File: create_main.c
 **
 **   Description: Creates widgets for the Xstation diagnostic monitor.
 **
 ****************************************************************************
 ****************************************************************************/

/************************************************************
*                                                           *
*  Revision history:                                        *
*                                                           *
*  02/09/91  Gary Goldman, IBM Austin   Initial Version     *
*                                                           *
*************************************************************/

#include "xmonitor.h"

/*****************************************************************************
 *                        Globals
 *****************************************************************************/
extern int help_index;
Widget main_form;

/*****************************************************************************
 *                        Locals
 *****************************************************************************/
static Widget main_window_widget;

/***************************************************************************
 * 							Function Declarations
 ***************************************************************************/
static void create_mainwindow();
extern void create_graphs();
extern void create_options();
extern void create_sampling();
extern void create_protocols();
extern void create_messages();
extern void create_report();
extern void create_help();

/***************************************************************************
 * Driver routine that creates all widgets.
 ***************************************************************************/
void create_monitor()
{
    create_mainwindow();
    create_options(); 

    /* Realize everything.  The interface comes up on the display now. */
    XtRealizeWidget(toplevel_widget);

    create_graphs();
    create_sampling(); 
	create_protocols();
	create_messages();
    create_report();
    create_help();
}

/***************************************************************************
 * Create main widgets like main_window and menu_bar.
 ***************************************************************************/
static void create_mainwindow()
{
    Widget menu_bar, file_entry, file_menu, option_entry, option_menu, 
		   graph_menu, help_entry, widget;
    Arg args[20];
    static char *endmesg = {"Xstation monitor exit"};
    XmString label;
	int i, n;

	/**********************************************************************
	 *					MainWindow and MenuBar
	 **********************************************************************/
	main_window_widget = XmCreateMainWindow(toplevel_widget, "MAIN_WINDOW",
						     				 NULL, 0);
	XtManageChild(main_window_widget);

	n = 0;
    XtSetArg(args[n], XmNorientation , XmHORIZONTAL); n++;
    XtSetArg(args[n],XmNspacing, 15); n++;
	menu_bar = XmCreateMenuBar(main_window_widget, "MENU_BAR", args, n);
	XtManageChild(menu_bar);

	/**********************************************************************
	 *							File
	 **********************************************************************/
	file_menu = XmCreatePulldownMenu (menu_bar, "FILE_MENU", NULL, 0);

	n = 0;
	label = XmStringLtoRCreate("Open...", XmSTRING_DEFAULT_CHARSET);
	XtSetArg(args[n], XmNlabelString, label); n++;
	XtSetArg(args[n], XmNmnemonic, 'O'); n++;
	XtSetArg(args[n], XmNsensitive, FALSE); n++;
	widget = XmCreatePushButtonGadget(file_menu, "OPEN", args, n);
	XtManageChild(widget);
    /*XtAddCallback(widget, XmNactivateCallback, open_template, NULL);*/

	n = 0;
	label = XmStringLtoRCreate("Save As...", XmSTRING_DEFAULT_CHARSET);
	XtSetArg(args[n], XmNlabelString, label); n++;
	XtSetArg(args[n], XmNmnemonic, 'S'); n++;
	XtSetArg(args[n], XmNsensitive, FALSE); n++;
	widget = XmCreatePushButtonGadget(file_menu, "SAVE", args, n);
	XtManageChild(widget);
    /*XtAddCallback(widget, XmNactivateCallback, save_template, NULL);*/

	n = 0;
	label = XmStringLtoRCreate("Exit", XmSTRING_DEFAULT_CHARSET);
	XtSetArg(args[n], XmNlabelString, label); n++;
	XtSetArg(args[n], XmNmnemonic, 'E'); n++;
	widget = XmCreatePushButtonGadget(file_menu, "EXIT", args, n);
	XtManageChild(widget);
    XtAddCallback(widget, XmNactivateCallback, exit_proc, endmesg);

	n = 0;
	label = XmStringLtoRCreate("File", XmSTRING_DEFAULT_CHARSET);
	XtSetArg(args[n], XmNlabelString, label); n++;
	XtSetArg(args[n], XmNmnemonic, 'F'); n++;
	XtSetArg(args[n], XmNsubMenuId, file_menu); n++;
	file_entry = XmCreateCascadeButton(menu_bar, "FILE_ENTRY", args, n);
	XtManageChild(file_entry);

	/**********************************************************************
	 *							Options	
	 **********************************************************************/
	option_menu = XmCreatePulldownMenu(menu_bar, "OPTION_MENU", NULL, 0);

	n = 0;
	label = XmStringLtoRCreate("Start Monitoring", XmSTRING_DEFAULT_CHARSET);
	XtSetArg(args[n], XmNlabelString, label); n++;
	XtSetArg(args[n], XmNmnemonic, 'S'); n++;
	widget = XmCreatePushButtonGadget(option_menu, "START", args, n); 
	XtManageChild(widget);
	XtAddCallback(widget, XmNactivateCallback, activate_monitor, NULL);

	graph_menu = XmCreatePulldownMenu(option_menu, "GRAPH_MENU", NULL, 0);

	for (i = 0; i < NUM_GRAPHS; i++) {
		n = 0;
		label = XmStringLtoRCreate(graph[i].title, XmSTRING_DEFAULT_CHARSET);
		XtSetArg(args[n], XmNlabelString, label); n++;
		widget = XmCreateCascadeButton(graph_menu, "DISP_GRAPH_B", args, n); 
		XtManageChild(widget);
		XtAddCallback(widget, XmNactivateCallback, display_graph, i);
	}

	n = 0;
	label = XmStringLtoRCreate("Display Graph", XmSTRING_DEFAULT_CHARSET);
	XtSetArg(args[n], XmNlabelString, label); n++;
	XtSetArg(args[n], XmNmnemonic, 'D'); n++;
	XtSetArg(args[n], XmNsubMenuId, graph_menu); n++;
	widget = XmCreateCascadeButton(option_menu, "DISPLAY_GRAPH", args, n); 
	XtManageChild(widget);
	XtAddCallback(widget, XmNactivateCallback, activate_monitor, NULL);

	n = 0;
	label = XmStringLtoRCreate("Options", XmSTRING_DEFAULT_CHARSET);
	XtSetArg(args[n], XmNlabelString, label); n++;
	XtSetArg(args[n], XmNmnemonic, 'O'); n++;
	XtSetArg(args[n], XmNsubMenuId, option_menu); n++;
	option_entry = XmCreateCascadeButton(menu_bar, "OPTION_ENTRY", args, n);
	XtManageChild(option_entry);

	/**********************************************************************
	 *								Help
	 **********************************************************************/
	n = 0;
	label = XmStringLtoRCreate("Help", XmSTRING_DEFAULT_CHARSET);
	XtSetArg(args[n], XmNlabelString, label); n++;
	XtSetArg(args[n], XmNmnemonic, 'H'); n++;
	help_entry = XmCreateCascadeButton(menu_bar, "MAIN_HELP", args, n);
	XtManageChild(help_entry);
	XtAddCallback(help_entry, XmNactivateCallback, activate_help, help_index);
	strcpy(help[help_index++].title, "Main Help");

	XtSetArg(args[0], XmNmenuHelpWidget, help_entry);
	XtSetValues (menu_bar, args, 1);

	/**********************************************************************
	 *							Main Form	
	 **********************************************************************/
	main_form = XmCreateForm(main_window_widget, "MAIN_FORM", NULL, 0);
	XtManageChild(main_form);

	XmStringFree(label);
}
