/****************************************************************************
 ****************************************************************************
 **
 **   File: create_opt_dialogs.c
 **
 **   Description: Creates all dialog widgets associated with the 
 **				   Xstation diagnostic monitor options.
 **
 ****************************************************************************
 ****************************************************************************/

/************************************************************
*                                                           *
*  Revision history:                                        *
*                                                           *
*  02/09/91  Gary Goldman, IBM Austin   Initial Version     *
*                                                           *
*************************************************************/

#include "xmonitor.h"

/*****************************************************************************
 *                        Globals
 *****************************************************************************/
extern int help_index;

/***************************************************************************
 * Create timed interval sampling rate dialog.
 ***************************************************************************/
void create_sampling()
{
    Widget button, widget;
    Arg args[20], default_arg;
    XmString label;
    int n, nsav;

    n = 0;
    label = XmStringLtoRCreate("Set Sampling Interval", 
								XmSTRING_DEFAULT_CHARSET);
    XtSetArg(args[n], XmNdialogTitle, label); n++;
    XtSetArg(args[n], XmNdefaultPosition, FALSE); n++;
    XtSetArg(args[n], XmNx, 1000); n++;
    XtSetArg(args[n], XmNy, 580); n++;
    XtSetArg(args[n], XmNautoUnmanage, False); n++;
    sample_rate_dialog = XmCreateFormDialog(toplevel_widget,
                                                 "SR_DIALOG", args, n);
    n = 0;
    XtSetArg(args[n], XmNfontList, widget_font); n++;
    XtSetArg(args[n], XmNdecimalPoints, 0); n++;
    XtSetArg(args[n], XmNmaximum, 10); n++;
    XtSetArg(args[n], XmNminimum, 0); n++;
    XtSetArg(args[n], XmNorientation, XmHORIZONTAL); n++;
    XtSetArg(args[n], XmNprocessingDirection, XmMAX_ON_RIGHT); n++;
    XtSetArg(args[n], XmNscaleHeight, 20); n++;
    XtSetArg(args[n], XmNshowValue, True); n++;
    XtSetArg(args[n], XmNtopAttachment, XmATTACH_FORM); n++;
    XtSetArg(args[n], XmNtopOffset, MARGIN); n++;
	nsav = n;
    label = XmStringLtoRCreate("0       minutes apart      10",
                                XmSTRING_DEFAULT_CHARSET);
    XtSetArg(args[n], XmNtitleString, label); n++;
    XtSetArg(args[n], XmNvalue, monitor_options.sample_time[0]); n++;
    XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg(args[n], XmNleftOffset, MARGIN); n++;
    sample_rate_scale[0] = XmCreateScale (sample_rate_dialog, "MINUTES_SCALE",
										  args, n);
    XtManageChild(sample_rate_scale[0]);

    n = nsav;
    label = XmStringLtoRCreate("0       seconds apart      60",
                                XmSTRING_DEFAULT_CHARSET);
    XtSetArg(args[n], XmNtitleString, label); n++;
    XtSetArg(args[n], XmNmaximum, 60); n++;
    XtSetArg(args[n], XmNvalue, monitor_options.sample_time[1]); n++;
    XtSetArg(args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
    XtSetArg(args[n], XmNleftWidget, sample_rate_scale[0]); n++;
    XtSetArg(args[n], XmNleftOffset, 20); n++;
    XtSetArg(args[n], XmNrightOffset, MARGIN); n++;
    sample_rate_scale[1] = XmCreateScale (sample_rate_dialog, "SECONDS_SCALE",
										  args, n);
    XtManageChild(sample_rate_scale[1]);

    n = 0;
    XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    XtSetArg(args[n], XmNtopWidget, sample_rate_scale[0]); n++;
    XtSetArg(args[n], XmNtopOffset, MARGIN); n++;
    XtSetArg(args[n], XmNbottomOffset, MARGIN); n++;
	nsav = n;
    label = XmStringLtoRCreate("OK", XmSTRING_DEFAULT_CHARSET);
    XtSetArg(args[n], XmNlabelString, label); n++;
    XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg(args[n], XmNleftOffset, MARGIN); n++;
    button = XmCreatePushButtonGadget(sample_rate_dialog, "SR_OK", args, n);
    XtManageChild(button);
    XtAddCallback(button, XmNactivateCallback, set_sample_interval, 0);

    XtSetArg(default_arg, XmNdefaultButton, button);
	XtSetValues(sample_rate_dialog, &default_arg, 1);

    n = nsav;
    XtSetArg(args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
    XtSetArg(args[n], XmNleftOffset, 5); n++;
	nsav = n;
    label = XmStringLtoRCreate("Apply", XmSTRING_DEFAULT_CHARSET);
    XtSetArg(args[n], XmNlabelString, label); n++;
    XtSetArg(args[n], XmNleftWidget, button); n++;
    button = XmCreatePushButtonGadget(sample_rate_dialog, "SR_APPLY", args, n);
    XtManageChild(button);
    XtAddCallback(button, XmNactivateCallback, set_sample_interval, 1);

    n = nsav;
    label = XmStringLtoRCreate("Cancel", XmSTRING_DEFAULT_CHARSET);
    XtSetArg(args[n], XmNlabelString, label); n++;
    XtSetArg(args[n], XmNleftWidget, button); n++;
    button = XmCreatePushButtonGadget(sample_rate_dialog, "SR_CANCEL", args, n);
    XtManageChild(button);
    XtAddCallback(button, XmNactivateCallback, set_sample_interval, 2);

    n = nsav-2;
    label = XmStringLtoRCreate("Help", XmSTRING_DEFAULT_CHARSET);
    XtSetArg(args[n], XmNlabelString, label); n++;
    XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
    XtSetArg(args[n], XmNrightOffset, MARGIN); n++;
    button = XmCreatePushButton(sample_rate_dialog, "SR_HELP", args, n);
    XtManageChild(button);
    XtAddCallback(button, XmNactivateCallback, activate_help, help_index);
	strcpy(help[help_index++].title, "Sample Rate Help");
}

/***************************************************************************
 * Create dialog that displays X protocol toggles.
 ***************************************************************************/
void create_protocols()
{
    Widget widget, scroll_w, rc, button;
    Arg args[20];
    XmString label;
	LINE *line_ptr;
	GRAPH *graph_ptr;
    int i, n, nsav = 0, line_idx;
    Dimension scroll_width = 225, scroll_height = 300;

    n = 0;
    label = XmStringLtoRCreate("X Protocols", XmSTRING_DEFAULT_CHARSET);
    XtSetArg(args[n], XmNdialogTitle, label); n++;
    XtSetArg(args[n], XmNwidth, scroll_width); n++;
    XtSetArg(args[n], XmNheight, scroll_height); n++;
    protocols_dialog = XmCreateFormDialog(toplevel_widget,
                                    "PROTOCOLS_DIALOG", args, n);
    n = 0;
    XtSetArg(args[n], XmNscrollBarDisplayPolicy, XmAS_NEEDED); n++;
    XtSetArg(args[n], XmNwidth, scroll_width); n++;
    XtSetArg(args[n], XmNheight, scroll_height); n++;
    XtSetArg(args[n], XmNscrollingPolicy, XmAUTOMATIC); n++;
    XtSetArg(args[n], XmNtopAttachment, XmATTACH_FORM); n++;
    XtSetArg(args[n], XmNtopOffset, MARGIN); n++;
    XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg(args[n], XmNleftOffset, MARGIN); n++;
    XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
    XtSetArg(args[n], XmNrightOffset, MARGIN); n++;
    XtSetArg(args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
    XtSetArg(args[n], XmNbottomOffset, BOTTOM_OFFSET); n++;
    scroll_w = XmCreateScrolledWindow(protocols_dialog, "PROTOCOLS_SCROLL",
									  args, n);
    XtManageChild(scroll_w);

    n = 0;
    XtSetArg(args[n], XmNnumColumns, 1); n++;
    XtSetArg(args[n], XmNpacking, XmPACK_COLUMN); n++;
    rc = XmCreateRowColumn (scroll_w, "PROTOCOLS_RC", args, n);
    XtManageChild(rc);

	graph_ptr = group[XPROTOCOL_GROUP].graph[0];
    XtSetArg(args[nsav], XmNset, TRUE); nsav++;
	FOR_LINES_IN_GRAPH_PTR(line_idx, graph_ptr) {
        line_ptr = graph_ptr->line[line_idx];
        n = nsav;
        label = XmStringLtoRCreate(line_ptr->name, XmSTRING_DEFAULT_CHARSET);
        XtSetArg(args[n], XmNlabelString, label); n++;
        XtSetArg(args[n], XmNselectColor, line_ptr->color); n++;
        widget = XmCreateToggleButtonGadget(rc, "PROTOCOL_TOGGLE", args, n);
        XtManageChild(widget);
        XtAddCallback(widget, XmNvalueChangedCallback, set_line_monitor,
                      line_ptr);
    }
    n = 0;
    label = XmStringLtoRCreate("Close", XmSTRING_DEFAULT_CHARSET);
    XtSetArg(args[n], XmNshowAsDefault, 1); n++;
    XtSetArg(args[n], XmNlabelString, label); n++;
    XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg(args[n], XmNleftOffset, MARGIN); n++;
    XtSetArg(args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
    XtSetArg(args[n], XmNbottomOffset, MARGIN); n++;
    button = XmCreatePushButtonGadget(protocols_dialog, "PROTOCOL_CLOSE", 
									  args, n);
    XtManageChild(button);
    XtAddCallback(button, XmNactivateCallback, close_protocols, NULL);

    XtSetArg(args[0], XmNdefaultButton, button);
	XtSetValues(protocols_dialog, args, 1);

    XmStringFree(label);
}
