/****************************************************************************
 ****************************************************************************
 **
 **   File: create_opt.c
 **
 **   Description: Creates Monitor Options widgets for Xstation diagnostic 
 **				   monitor.
 **
 ****************************************************************************
 ****************************************************************************/

/************************************************************
*                                                           *
*  Revision history:                                        *
*                                                           *
*  02/09/91  Gary Goldman, IBM Austin   Initial Version     *
*                                                           *
*************************************************************/

#include "xmonitor.h"

/*****************************************************************************
 *                        		Globals
 *****************************************************************************/
extern Widget main_form;

/*****************************************************************************
 *                        		Locals
 *****************************************************************************/
static Widget main_window_widget;
static int help_index = 0;

/***************************************************************************
 * Create monitor option widget.
 ***************************************************************************/
void create_options()
{
	Widget frame, rc, form, text, opt_menu, submenu, popup, button, widget;
	Arg args [20];
	XmString label;
	char text_value[7];
	LINE *line_ptr;
	GRAPH *graph_ptr;
	int i, n, nsav, h_offset = 10, v_offset = 15, line_idx, graph_idx;

	submenu = XmCreatePulldownMenu (main_form, "SERVERSM", NULL, 0);

	for (i = 0; i < monitor_options.server.num; i++) {
		n = 0;
		label = XmStringLtoRCreate(monitor_options.server.names[i], 
								   XmSTRING_DEFAULT_CHARSET);
		XtSetArg(args[n], XmNlabelString, label); n++;
		button = XmCreatePushButtonGadget(submenu, "SERVERPB", args, n);
		XtManageChild(button);
		XtAddCallback(button, XmNactivateCallback, set_server, i);
	}
	n = 0;
	label = XmStringLtoRCreate("Server to Sample: ", XmSTRING_DEFAULT_CHARSET);
	XtSetArg(args[n], XmNlabelString, label); n++;
	XtSetArg(args[n], XmNsubMenuId, submenu); n++;
	XtSetArg(args[n], XmNmarginWidth, 1); n++;
	XtSetArg(args[n], XmNtopAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNtopOffset, v_offset); n++;
	XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNleftOffset, h_offset); n++;
	widget = XmCreateOptionMenu (main_form, "SERVEROMEN", args, n);
	XtManageChild(widget);

	n = 0;
	label = XmStringLtoRCreate("Sample Count: ", XmSTRING_DEFAULT_CHARSET);
	XtSetArg(args[n], XmNlabelString, label); n++;
	XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
	XtSetArg(args[n], XmNtopWidget, widget); n++;
	XtSetArg(args[n], XmNtopOffset, v_offset); n++;
	XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNleftOffset, h_offset); n++;
	widget = XmCreateLabelGadget (main_form, "SCLABEL", args, n);
	XtManageChild(widget);

	n = 0;
	XtSetArg(args[n], XmNshadowType, XmSHADOW_IN); n++;
	XtSetArg(args[n], XmNtopAttachment, XmATTACH_OPPOSITE_WIDGET); n++;
	XtSetArg(args[n], XmNtopWidget, widget); n++;
	XtSetArg(args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
	XtSetArg(args[n], XmNleftWidget, widget); n++;
	XtSetArg(args[n], XmNbottomAttachment, XmATTACH_OPPOSITE_WIDGET); n++;
	XtSetArg(args[n], XmNbottomWidget, widget); n++;
	frame = XmCreateFrame (main_form, "SCFRAME", args, n);
	XtManageChild(frame);

	n = 0;
	label = XmStringLtoRCreate("0", XmSTRING_DEFAULT_CHARSET);
	XtSetArg(args[n], XmNlabelString, label); n++;
	sample_count_label = XmCreateLabelGadget (frame, "SCOUNT", args, n);
	XtManageChild(sample_count_label);

	submenu = XmCreatePulldownMenu (main_form, "SRATEOPTM", NULL, 0);

	n = 0;
	label = XmStringLtoRCreate("On Demand", XmSTRING_DEFAULT_CHARSET);
	XtSetArg(args[n], XmNlabelString, label); n++;
	widget = XmCreatePushButtonGadget(submenu, "ONDEMAND", args, n);
	XtManageChild(widget);
	XtAddCallback(widget, XmNactivateCallback, set_sample_rate, 0);

	n = 0;
	label = XmStringLtoRCreate("Timed Interval", XmSTRING_DEFAULT_CHARSET);
	XtSetArg(args[n], XmNlabelString, label); n++;
	widget = XmCreatePushButtonGadget(submenu, "TINTERVAL", args, n);
	XtManageChild(widget);
	XtAddCallback(widget, XmNactivateCallback, set_sample_rate, 1);

	n = 0;
	label = XmStringLtoRCreate("Sampling Rate: ", XmSTRING_DEFAULT_CHARSET);
	XtSetArg(args[n], XmNlabelString, label); n++;
	XtSetArg(args[n], XmNsubMenuId, submenu); n++;
	XtSetArg(args[n], XmNmarginWidth, 1); n++;
	XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
	XtSetArg(args[n], XmNtopWidget, frame); n++;
	XtSetArg(args[n], XmNtopOffset, v_offset); n++;
	XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNleftOffset, h_offset); n++;
	widget = XmCreateOptionMenu (main_form, "TFREQOMENU", args, n);
	XtManageChild(widget);

	n = 0;
	label = XmStringLtoRCreate("Request History Size: ", 
							   XmSTRING_DEFAULT_CHARSET);
	XtSetArg(args[n], XmNlabelString, label); n++;
	XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
	XtSetArg(args[n], XmNtopWidget, widget); n++;
	XtSetArg(args[n], XmNtopOffset, v_offset); n++;
	XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNleftOffset, h_offset); n++;
	widget = XmCreateLabelGadget (main_form, "RHSLABEL", args, n);
	XtManageChild(widget);

	n = 0;
	XtSetArg(args[n], XmNcolumns, 6); n++;
	XtSetArg(args[n], XmNmaxLength, 6); n++;
	sprintf(text_value, "%d\0", monitor_options.history_size);
	XtSetArg(args[n], XmNvalue, text_value); n++;
	XtSetArg(args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
	XtSetArg(args[n], XmNleftWidget, widget); n++;
	XtSetArg(args[n], XmNleftOffset, h_offset); n++;
	XtSetArg(args[n], XmNbottomAttachment, XmATTACH_OPPOSITE_WIDGET); n++;
	XtSetArg(args[n], XmNbottomWidget, widget); n++;
	text = XmCreateText(main_form, "RHSTEXT", args, n);
	XtManageChild(text);
	XtAddCallback(text, XmNmodifyVerifyCallback, set_history_size, NULL);

	submenu = XmCreatePulldownMenu (main_form, "CFILSM", NULL, 0);

	n = 0;
	label = XmStringLtoRCreate("On", XmSTRING_DEFAULT_CHARSET);
	XtSetArg(args[n], XmNlabelString, label); n++;
	XtSetArg(args[n], XmNsensitive, FALSE); n++;
	client_filter_widget[0] = XmCreatePushButtonGadget(submenu, "CFON", 
				args, n);
	XtManageChild(client_filter_widget[0]);
	XtAddCallback(client_filter_widget[0], XmNactivateCallback, 
				  set_client_filter, 0);
	n = 0;
	label = XmStringLtoRCreate("Off", XmSTRING_DEFAULT_CHARSET);
	XtSetArg(args[n], XmNlabelString, label); n++;
	XtSetArg(args[n], XmNsensitive, FALSE); n++;
	client_filter_widget[1] = XmCreatePushButtonGadget(submenu, "CFOFF", 
				args, n);
	XtManageChild(client_filter_widget[1]);
	XtAddCallback(client_filter_widget[1], XmNactivateCallback, 
				set_client_filter, 1);
	n = 0;
	label = XmStringLtoRCreate("Set", XmSTRING_DEFAULT_CHARSET);
	XtSetArg(args[n], XmNlabelString, label); n++;
	XtSetArg(args[n], XmNsensitive, FALSE); n++;
	widget = XmCreatePushButtonGadget(submenu, "CFSET", args, n);
	XtManageChild(widget);
	XtAddCallback(widget, XmNactivateCallback, set_client_filter, 2);

	n = 0;
	label = XmStringLtoRCreate("Client Filtering: ", XmSTRING_DEFAULT_CHARSET);
	XtSetArg(args[n], XmNlabelString, label); n++;
	XtSetArg(args[n], XmNsubMenuId, submenu); n++;
	XtSetArg(args[n], XmNmarginWidth, 1); n++;
	XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
	XtSetArg(args[n], XmNtopWidget, text); n++;
	XtSetArg(args[n], XmNtopOffset, v_offset); n++;
	XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNleftOffset, h_offset); n++;
	client_filter_widget[2] = XmCreateOptionMenu (main_form, "CFOMEN", args, n);
	XtManageChild(client_filter_widget[2]);

	submenu = XmCreatePulldownMenu (main_form, "GRPARSM", NULL, 0);

	FOR_ALL_GRAPHS(i) {
		n = 0;
		label = XmStringLtoRCreate(graph[i].title, XmSTRING_DEFAULT_CHARSET);
		XtSetArg(args[n], XmNlabelString, label); n++;
		XtSetArg(args[n], XmNsensitive, FALSE); n++;
		widget = XmCreatePushButtonGadget(submenu, "GRPARPB", args, n);
		XtManageChild(widget);
	}

	n = 0;
	label = XmStringLtoRCreate("Graph Parameters: ", XmSTRING_DEFAULT_CHARSET);
	XtSetArg(args[n], XmNlabelString, label); n++;
	XtSetArg(args[n], XmNsubMenuId, submenu); n++;
	XtSetArg(args[n], XmNmarginWidth, 1); n++;
	XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
	XtSetArg(args[n], XmNtopWidget, client_filter_widget[2]); n++;
	XtSetArg(args[n], XmNtopOffset, v_offset); n++;
	XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNleftOffset, h_offset); n++;
	XtSetArg(args[n], XmNrightOffset, h_offset); n++;
	opt_menu = XmCreateOptionMenu (main_form, "GRPOMEN", args, n);
	XtManageChild(opt_menu);

	submenu = XmCreatePulldownMenu (main_form, "SLOCKSM", NULL, 0);

	n = 0;
	label = XmStringLtoRCreate("On", XmSTRING_DEFAULT_CHARSET);
	XtSetArg(args[n], XmNlabelString, label); n++;
	XtSetArg(args[n], XmNsensitive, FALSE); n++;
	widget = XmCreatePushButtonGadget(submenu, "SLON", args, n);
	XtManageChild(widget);
	XtAddCallback(widget, XmNactivateCallback, set_scroll_lock, TRUE);
	n = 0;
	label = XmStringLtoRCreate("Off", XmSTRING_DEFAULT_CHARSET);
	XtSetArg(args[n], XmNlabelString, label); n++;
	XtSetArg(args[n], XmNsensitive, FALSE); n++;
	widget = XmCreatePushButtonGadget(submenu, "SLOFF", args, n);
	XtManageChild(widget);
	XtAddCallback(widget, XmNactivateCallback, set_scroll_lock , FALSE);

	n = 0;
	label = XmStringLtoRCreate("Scroll Lock: ", XmSTRING_DEFAULT_CHARSET);
	XtSetArg(args[n], XmNlabelString, label); n++;
	XtSetArg(args[n], XmNsubMenuId, submenu); n++;
	XtSetArg(args[n], XmNmarginWidth, 1); n++;
	XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
	XtSetArg(args[n], XmNtopWidget, opt_menu); n++;
	XtSetArg(args[n], XmNtopOffset, v_offset); n++;
	XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNleftOffset, h_offset); n++;
	opt_menu = XmCreateOptionMenu (main_form, "SLOMEN", args, n);
	XtManageChild(opt_menu);

	n = 0;
	XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
	XtSetArg(args[n], XmNtopWidget, opt_menu); n++;
	XtSetArg(args[n], XmNtopOffset, v_offset); n++;
	XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNleftOffset, h_offset); n++;
	rc = XmCreateRowColumn (main_form, "RPTRC", args, n);
	XtManageChild(rc);

	n = 0;
	label = XmStringLtoRCreate("Report", XmSTRING_DEFAULT_CHARSET);
	XtSetArg(args[n], XmNlabelString, label); n++;
	widget = XmCreatePushButtonGadget(rc, "RPTPB", args, n);
	XtManageChild(widget);

	popup = XmCreatePopupMenu(rc, "RPTPOPUP", NULL, 0);
	XtAddEventHandler(rc, ButtonPressMask, False, post_report, popup);

	n = 0;
	label = XmStringLtoRCreate("Generate Report", XmSTRING_DEFAULT_CHARSET);
	XtSetArg(args[n], XmNlabelString, label); n++;
	widget = XmCreatePushButtonGadget(popup, "RPTBUTTON", args, n);
	XtManageChild(widget);
	XtAddCallback(widget, XmNactivateCallback, gen_report, NULL);

	n = 0;
	label = XmStringLtoRCreate("Display Report", XmSTRING_DEFAULT_CHARSET);
	XtSetArg(args[n], XmNlabelString, label); n++;
	widget = XmCreatePushButtonGadget (popup, "RPTBUTTON", args, n);
	XtManageChild(widget);
	XtAddCallback(widget, XmNactivateCallback, display_report, NULL);

	n = 0;
	XtSetArg(args[n], XmNshadowType, XmSHADOW_IN); n++;
	XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
	XtSetArg(args[n], XmNtopWidget, rc); n++;
	XtSetArg(args[n], XmNtopOffset, v_offset); n++;
	XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNleftOffset, h_offset); n++;
	XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNrightOffset, h_offset); n++;
	frame = XmCreateFrame (main_form, "RESFRAME", args, n);
	XtManageChild(frame);

	n = 0;
	XtSetArg(args[n], XmNnumColumns, 1); n++;
	XtSetArg(args[n], XmNpacking, XmPACK_COLUMN); n++;
	rc = XmCreateRowColumn (frame, "RESRC", args, n);
	XtManageChild(rc);

	n = 0;
	label = XmStringLtoRCreate("Resources to Monitor", 
			XmSTRING_DEFAULT_CHARSET);
	XtSetArg(args[n], XmNlabelString, label); n++;
	XtSetArg(args[n], XmNfontList, label_font); n++;
	widget = XmCreateLabelGadget(rc, "RESLABEL", args, n);
	XtManageChild(widget);

	nsav = 0;
	XtSetArg(args[nsav], XmNset, TRUE); nsav++;
	FOR_GRAPHS_IN_GROUP(graph_idx, RESOURCE_GROUP) {
		graph_ptr = group[RESOURCE_GROUP].graph[graph_idx];
		FOR_LINES_IN_GRAPH_PTR(line_idx, graph_ptr) {
			line_ptr = graph_ptr->line[line_idx];
			n = nsav;
			label = XmStringLtoRCreate(line_ptr->name, 
					XmSTRING_DEFAULT_CHARSET);
			XtSetArg(args[n], XmNlabelString, label); n++;
			XtSetArg(args[n], XmNselectColor, line_ptr->color); n++;
			widget = XmCreateToggleButtonGadget(rc, "RESTOGGLE", args, n);
			XtManageChild(widget);
			XtAddCallback(widget, XmNvalueChangedCallback, set_line_monitor, 
					line_ptr);
		}
    }

	n = 0;
	label = XmStringLtoRCreate("X Protocols", XmSTRING_DEFAULT_CHARSET);
	XtSetArg(args[n], XmNlabelString, label); n++;
	XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
	XtSetArg(args[n], XmNtopWidget, frame); n++;
	XtSetArg(args[n], XmNtopOffset, v_offset); n++;
	XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNleftOffset, h_offset); n++;
	widget = XmCreatePushButtonGadget (main_form, "XPROTOCOLS", args, n);
	XtManageChild(widget);
	XtAddCallback(widget, XmNactivateCallback, activate_protocols, NULL);

	n = 0;
	XtSetArg(args[n], XmNshadowType, XmSHADOW_IN); n++;
	XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
	XtSetArg(args[n], XmNtopWidget, widget); n++;
	XtSetArg(args[n], XmNtopOffset, v_offset); n++;
	XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNleftOffset, h_offset); n++;
	XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNrightOffset, h_offset); n++;
	XtSetArg(args[n], XmNbottomOffset, v_offset); n++;
	frame = XmCreateFrame (main_form, "NETFRAME", args, n);
	XtManageChild(frame);

	n = 0;
	XtSetArg(args[n], XmNnumColumns, 1); n++;
	XtSetArg(args[n], XmNpacking, XmPACK_COLUMN); n++;
	rc = XmCreateRowColumn (frame, "NETRC", args, n);
	XtManageChild(rc);

	n = 0;
	label = XmStringLtoRCreate("Network Monitoring", XmSTRING_DEFAULT_CHARSET);
	XtSetArg(args[n], XmNlabelString, label); n++;
	XtSetArg(args[n], XmNfontList, label_font); n++;
	widget = XmCreateLabelGadget(rc, "NETLABEL", args, n);
	XtManageChild(widget);

	n = 0;
	label = XmStringLtoRCreate("% idle", XmSTRING_DEFAULT_CHARSET);
	XtSetArg(args[n], XmNlabelString, label); n++;
	XtSetArg(args[n], XmNsensitive, FALSE); n++;
	widget = XmCreateToggleButton(rc, "NETTOGGLE", args, n);
	XtManageChild(widget);

	XmStringFree(label);
}
