/****************************************************************************
 ****************************************************************************
 **
 **   File: methods.c
 **
 **   Description: Defines method functions for Xstation monitor.
 **
 ****************************************************************************
 ****************************************************************************/

/************************************************************
*                                                           *
*  Revision history:                                        *
*                                                           *
*  02/02/91  Gary Goldman, IBM Austin                       *
*                                                           *
*************************************************************/

#include "xmonitor.h"
#include "xsstat.h"

#define FIELDS_IN_LINE 8

/***************************************************************************
 * Method to extract values from server dependent structure.
 ***************************************************************************/
void get_resources(statistics, graph_num, current)
	struct SSTATinfo statistics;
	int graph_num;
	int current;
{
	switch (graph_num) {
		case 0:
			graph[graph_num].line[0]->value[current] = statistics.GCCount;
			graph[graph_num].line[1]->value[current] = statistics.WinCount;
			graph[graph_num].line[2]->value[current] = statistics.PixmapCount;
			break;
		case 1:
			graph[graph_num].line[0]->value[current] = statistics.TotMem;
			graph[graph_num].line[1]->value[current] = 
					statistics.TotMem-statistics.FreeMem;
			graph[graph_num].line[2]->value[current] = statistics.BigFrag;
			break;
		case 2:
			graph[graph_num].line[0]->value[current] = statistics.FragCount;
			break;
	}
}

/***************************************************************************
 * Method to extract values from server dependent structure.
 ***************************************************************************/
void get_protocols(statistics, graph_num, current)
	struct SSTATinfo statistics;
	int graph_num;
	int current;
{
	int i, line_idx, protocol_id;

	FOR_LINES_IN_GRAPH(line_idx, graph_num)
			graph[graph_num].line[line_idx]->value[current] = 0;
    for (i = 0; i < statistics.NumClientReq; i++) {
		protocol_id = statistics.ClientReq[i].Xrequest;
		/* look for this line attribute in each line of protocol graph */
		FOR_LINES_IN_GRAPH(line_idx, graph_num) {
			if (graph[graph_num].line[line_idx]->id == protocol_id) {
        		graph[graph_num].line[line_idx]->value[current]++;
				break;
			}
        }
    }
}

/***************************************************************************
 * Prints report for RESOURCES.
 ***************************************************************************/
void rpt_resources(rpt_file)
	FILE *rpt_file;
{
	extern int find_max();
	extern char *asterick();
	LINE *line_ptr;
	GRAPH *graph_ptr;
	int j, start, end, num_values, max_value, field_count, line_idx,
		graph_idx, special_case[] = {1,0};

	FOR_GRAPHS_IN_GROUP(graph_idx, RESOURCE_GROUP) {
		graph_ptr = group[RESOURCE_GROUP].graph[graph_idx];
        fprintf(rpt_file,
            "\n\n                           %s\n\n", graph_ptr->title);
		FOR_LINES_IN_GRAPH_PTR(line_idx, graph_ptr) {
			line_ptr = graph_ptr->line[line_idx];
	        start = line_ptr->start_of_data;
    	    end = graph_ptr->view.current;
        	num_values = end-start+1;
        	max_value = find_max(&line_ptr->value[start], num_values);
        	fprintf(rpt_file, "%s    (max = %d)\n", line_ptr->name,
            	        max_value);
			/* check for total memory */
			if (strcmp(graph_ptr->title, "Memory Statistics") || line_idx) {
        		field_count = 0;
        		for (j = start; j <= end; j++) {
            		/* check for max */
            		if (line_ptr->value[j] == max_value)
                		fprintf(rpt_file, "%9s", asterick(line_ptr->value[j]));
            		else
                		fprintf(rpt_file, "%9d", line_ptr->value[j]);
            		if (++field_count == FIELDS_IN_LINE) {
                		fprintf(rpt_file,"\n");
                		field_count = 0;
            		}
				}
        	}
        	fprintf(rpt_file,"\n");
    	}
	}
}

/***************************************************************************
 * Prints report for XPROTOCOLS.
 ***************************************************************************/
void rpt_protocols(rpt_file)
	FILE *rpt_file;
{
	extern int find_max();
	extern char *asterick();
	LINE *line_ptr;
	GRAPH *graph_ptr;
	int j, start, end, num_values, max_value, field_count, line_idx,
		graph_idx = 0;
	struct {
		LINE *line;
		int  max;
	} maximums[NUM_PROTOCOLS];
	int max_sorted[NUM_PROTOCOLS], count = 0, comp();

	graph_ptr = group[XPROTOCOL_GROUP].graph[graph_idx];
    fprintf(rpt_file,
           "\n\n                           %s\n\n", graph_ptr->title);
	FOR_LINES_IN_GRAPH_PTR(line_idx, graph_ptr) {
		line_ptr = graph_ptr->line[line_idx];
        start = line_ptr->start_of_data;
   	    end = graph_ptr->view.current;
       	num_values = end-start+1;
       	max_value = find_max(&line_ptr->value[start], num_values);
		maximums[count].line = line_ptr;    /* to sort latter */
		maximums[count].max = max_value; 	/*		"     "   */
		max_sorted[count++] = max_value;    /*      "     "   */
		if (max_value) {  /* if no values to report don't print */
        	fprintf(rpt_file, "%s    (max = %d)\n", line_ptr->name,
           		    max_value);
        	field_count = 0;
   	    	for (j = start; j <= end; j++) {
       	    	/* check for max */
           		if (line_ptr->value[j] == max_value)
               		fprintf(rpt_file, "%9s", asterick(line_ptr->value[j]));
           		else
               		fprintf(rpt_file, "%9d", line_ptr->value[j]);
           		if (++field_count == FIELDS_IN_LINE) {
               		fprintf(rpt_file,"\n");
               		field_count = 0;
           		}
			}
       		fprintf(rpt_file,"\n");
       	}
   	}

	/* give sorted count of protocols */
	fprintf(rpt_file, "\nSORTED PROTOCOLS\n\n");
	qsort(max_sorted, NUM_PROTOCOLS, sizeof(int), comp);
	for (count = 0; count < NUM_PROTOCOLS; count++)
		if (max_sorted[count])
			for (j = 0; j < NUM_PROTOCOLS; j++)  /* find 1st occur of value */
				if (maximums[j].max == max_sorted[count]) {
					fprintf(rpt_file,"%-30s  %3d\n", maximums[j].line->name, 
							maximums[j].max);
					maximums[j].max = 0;  /* so it won't be picked up again */
				}
}

int comp(i, j)
    int *i, *j;
{
    return(*j-*i);
}
