/****************************************************************************
 ****************************************************************************
 **
 **   File: read_template.c
 **
 **   Description: Reads in group, graph and option descriptions from file.
 **
 ****************************************************************************
 ****************************************************************************/

/************************************************************
*                                                           *
*  Revision history:                                        *
*                                                           *
*  01/17/90  Gary Goldman, IBM Austin                       *
*                                                           *
*************************************************************/

#include "xmonitor.h"

/*****************************************************************************
 *          		Function Declarations
 *****************************************************************************/
static void set_groups();
static void set_graphs();
static void set_lines();
static void set_labels();
static void set_options();

/***************************************************************************
 * Initialize all defaults if a template file not specified 
 * 	  (active and temporary).
 ***************************************************************************/
void read_template(filename)
	char *filename;
{
	set_groups();
	set_graphs(); /* graphs, and lines */
	set_options();
}

/***************************************************************************
 * Set highest logical parenting (groups).
 ***************************************************************************/
static void set_groups()
{ 
    group[RESOURCE_GROUP].num_graphs = 3;
    group[XPROTOCOL_GROUP].num_graphs = 1;

	group[RESOURCE_GROUP].graph = (GRAPH **) 
			malloc(sizeof(GRAPH **)*group[RESOURCE_GROUP].num_graphs);
	group[XPROTOCOL_GROUP].graph = (GRAPH **)
			malloc(sizeof(GRAPH **)*group[XPROTOCOL_GROUP].num_graphs);
    group[RESOURCE_GROUP].graph[0] = &graph[0];
    group[RESOURCE_GROUP].graph[1] = &graph[1];
    group[RESOURCE_GROUP].graph[2] = &graph[2];
	group[XPROTOCOL_GROUP].graph[0] = &graph[3];

    group[RESOURCE_GROUP].title = (char *)malloc(10);
    strcpy(group[RESOURCE_GROUP].title, "RESOURCES");
    group[XPROTOCOL_GROUP].title = (char *)malloc(12);
    strcpy(group[XPROTOCOL_GROUP].title, "X PROTOCOLS");

	group[RESOURCE_GROUP].value_method = get_resources;
	group[XPROTOCOL_GROUP].value_method = get_protocols;
	group[RESOURCE_GROUP].rpt_method = rpt_resources;
	group[XPROTOCOL_GROUP].rpt_method = rpt_protocols;
}

/***************************************************************************
 * Initialize all defaults if a template file not specified 
 * 	  (active and temporary).
 ***************************************************************************/
static void set_graphs()
{
	unsigned long valuemask = GCForeground | GCLineWidth;
	XGCValues values;
	int group_num, graph_num, graph_idx, line_idx, line_count = 0;
	GRAPH *graph_ptr;

	LOOKUP_GRAPH(group_num, graph_idx, graph_num) {
		if (&graph[graph_num] == group[group_num].graph[graph_idx]) {
			graph[graph_num].group = &group[group_num];
			break;
		}
	}

	/* position viewport in lower right hand corner */
	graph_ptr = group[RESOURCE_GROUP].graph[0];
	graph_ptr->title = (char *)malloc(15);
	strcpy(graph_ptr->title, "Resource Count");
	graph_ptr->background = white;
	graph_ptr->geom.position.y = 0;
	graph_ptr->geom.inc_pixels.x = 10;
	graph_ptr->geom.inc_pixels.y = 10;
	graph_ptr->geom.inc_units.x = 1;
	graph_ptr->geom.inc_units.y = 10;
	graph_ptr->geom.max_inc.x = 100;
	graph_ptr->geom.max_inc.y = 60;
	graph_ptr->view.size.x = 50;
	graph_ptr->view.size.y = 20;
	graph_ptr->num_lines = 3;
	graph_ptr->line = (LINE **) malloc(sizeof(LINE **)*graph_ptr->num_lines);
	graph_ptr->line[0] = &line[line_count++];
	graph_ptr->line[1] = &line[line_count++];
	graph_ptr->line[2] = &line[line_count++];

	graph_ptr = group[RESOURCE_GROUP].graph[1];
	graph_ptr->title = (char *)malloc(18);
	strcpy(graph_ptr->title, "Memory Statistics");
	graph_ptr->background = white;
	graph_ptr->geom.position.y = 318;
	graph_ptr->geom.inc_pixels.x = 10;
	graph_ptr->geom.inc_pixels.y = 20;
	graph_ptr->geom.inc_units.x = 1;
	graph_ptr->geom.inc_units.y = 1000000;
	graph_ptr->geom.max_inc.x = 100;
	graph_ptr->geom.max_inc.y = 10;
	graph_ptr->view.size.x = 50;
	graph_ptr->view.size.y = 6;
	graph_ptr->num_lines = 3;
	graph_ptr->line = (LINE **) malloc(sizeof(LINE **)*graph_ptr->num_lines);
	graph_ptr->line[0] = &line[line_count++];
	graph_ptr->line[1] = &line[line_count++];
	graph_ptr->line[2] = &line[line_count++];

	graph_ptr = group[RESOURCE_GROUP].graph[2];
	graph_ptr->title = (char *)malloc(25);
	strcpy(graph_ptr->title, "Free List Fragment Count");
	graph_ptr->background = white;
	graph_ptr->geom.position.y = 556;
	graph_ptr->geom.inc_pixels.x = 10;
	graph_ptr->geom.inc_pixels.y = 10;
	graph_ptr->geom.inc_units.x = 1;
	graph_ptr->geom.inc_units.y = 10;
	graph_ptr->geom.max_inc.x = 100;
	graph_ptr->geom.max_inc.y = 25;
	graph_ptr->view.size.x = 50;
	graph_ptr->view.size.y = 15;
	graph_ptr->num_lines = 1;
	graph_ptr->line = (LINE **) malloc(sizeof(LINE **)*graph_ptr->num_lines);
	graph_ptr->line[0] = &line[line_count++];

	graph_ptr = group[XPROTOCOL_GROUP].graph[0];
	graph_ptr->title = (char *)malloc(12);
	strcpy(graph_ptr->title, "X Protocols");
	graph_ptr->background = white;
	graph_ptr->geom.position.y = 826;
	graph_ptr->geom.inc_pixels.x = 10;
	graph_ptr->geom.inc_pixels.y = 30;
	graph_ptr->geom.inc_units.x = 1;
	graph_ptr->geom.inc_units.y = 10;
	graph_ptr->geom.max_inc.x = 100;
	graph_ptr->geom.max_inc.y = 3;
	graph_ptr->view.size.x = 50;
	graph_ptr->view.size.y = 2;
	graph_ptr->num_lines = NUM_PROTOCOLS;
	graph_ptr->line = (LINE **) 
			malloc(sizeof(LINE **)*graph_ptr->num_lines);
	for (line_idx = 0; line_idx < NUM_PROTOCOLS; line_idx++) 
		graph_ptr->line[line_idx] = &line[line_count++];

	/* go back and set everything based on previous initializations */
	FOR_ALL_GRAPHS(graph_num) {
		values.foreground = graph[graph_num].background; /* for gc seting */
		graph[graph_num].geom.position.x = 0;
		graph[graph_num].view.end.x = graph[graph_num].geom.max_inc.x;
		graph[graph_num].view.start.x = 
				graph[graph_num].view.end.x-graph[graph_num].view.size.x;
		graph[graph_num].view.start.y = 0;
		graph[graph_num].view.end.y = 
				graph[graph_num].view.start.y+graph[graph_num].view.size.y;
		graph[graph_num].view.current = graph[graph_num].view.start.x-1;
		graph[graph_num].labels[0].inc = 5;
		graph[graph_num].labels[1].inc = 5;
		set_labels(graph_num);
	}

	/* set LINES */
	set_lines();

	/* go back and set graph stuff based on line initializations */
	FOR_ALL_GRAPHS(graph_num) {
		values.line_width = graph[graph_num].line_width; /* for line erasing */
		graph[graph_num].gc = XCreateGC(dpy, DefaultRootWindow(dpy), valuemask, 
				&values);
	}
}

/***************************************************************************
 * Initialize the line structure.
 ***************************************************************************/
static void set_lines()
{
	FILE *attrib_file;
	char color_name[30];
	XColor exact, color;
	unsigned long valuemask = GCForeground | GCLineWidth;
	XGCValues values;
	int line_idx, graph_num, line_num = 0;
	GRAPH *graph_ptr;

	FOR_ALL_GRAPHS(graph_num) 
		FOR_LINES_IN_GRAPH(line_idx, graph_num)
			line[line_num++].graph = &graph[graph_num];

    if ((attrib_file = fopen("line_attributes", "r")) == NULL) {
        printf("Cannot open line_attributes file\n");
        exit (0);
	}

	FOR_ALL_LINES(line_num) {
		line[line_num].monitor = TRUE;
        fscanf (attrib_file, "%s%d%s%d", line[line_num].name, 
				&line[line_num].id, color_name, &line[line_num].width);
    	XAllocNamedColor(dpy, cmap, color_name, &color, &exact);
		values.foreground = line[line_num].color = color.pixel;
		values.line_width = line[line_num].width;
		line[line_num].gc = XCreateGC(dpy, DefaultRootWindow(dpy), valuemask, 
						 &values);
		graph_ptr = line[line_num].graph;
		line[line_num].value = (int *) malloc(sizeof(int)*
								(graph_ptr->geom.max_inc.x+1));
		line[line_num].point = (XPoint *) malloc(sizeof(XPoint)*
								(graph_ptr->geom.max_inc.x+1));
		line[line_num].start_of_data = graph_ptr->view.start.x;
		/* for variable width lines latter - erase using widest line */
		if (line[line_num].width > graph_ptr->line_width)
			graph_ptr->line_width = line[line_num].width;
	}
	fclose(attrib_file);
}

/***************************************************************************
 * Initialize the y axis labels.
 ***************************************************************************/
static void set_labels(graph_num)
	int graph_num;
{
	int i;

	for (i = 0; i < 2; i++) {
		graph[graph_num].labels[i].num = graph[graph_num].labels[i].num = 
			(graph[graph_num].view.size.y/graph[graph_num].labels[i].inc)+1;
		graph[graph_num].labels[i].value = 
			(int *) malloc(sizeof(int)*graph[graph_num].labels[i].num);
		graph[graph_num].labels[i].widget = 
			(Widget *) malloc(sizeof(Widget)*graph[graph_num].labels[i].num);
	}
}

/***************************************************************************
 * Set the y axis label's values.
 ***************************************************************************/
void set_graph_labels(graph_num)
	int graph_num;
{
	Arg label_arg;
	XmString label;
	char digits[33];
	int i, side;

	for (side = 0; side < 2; side++) {
		for (i = 0; i < graph[graph_num].labels[side].num; i++) {
			graph[graph_num].labels[side].value[i] = 
				((i*graph[graph_num].labels[side].inc)+
					graph[graph_num].view.start.y)*
						graph[graph_num].geom.inc_units.y;
        	(graph[graph_num].labels[side].value[i] > 999) ?
            	sprintf(digits, "%g\0", 
					(float)graph[graph_num].labels[side].value[i]) :
            	sprintf(digits, "%d\0", graph[graph_num].labels[side].value[i]);
        	label = XmStringLtoRCreate(digits, XmSTRING_DEFAULT_CHARSET);
        	XtSetArg(label_arg, XmNlabelString, label);
			XtSetValues(graph[graph_num].labels[side].widget[i], &label_arg, 1);
		}	
	}
	XmStringFree(label);
}

/***************************************************************************
 * Initialize the monitor options.
 ***************************************************************************/
static void set_options()
{
	FILE *server_file;
	char server_name[30];
	int i;

	monitor_options.sample_rate = ON_DEMAND;
	monitor_options.sample_time[0] = 0;
	monitor_options.sample_time[1] = 2;
	monitor_options.history_size = 30; /* CHANGE HERE FOR BUG */
	monitor_options.client_filter = FALSE;
	monitor_options.slock = TRUE;

    if ((server_file = fopen("server.list", "r")) == NULL) {
        printf("Cannot open server.list file\n");
        exit (0);
	}
	i = -1;
	while (i < MAX_SERVERS) {
        fscanf (server_file, "%s", server_name);
		if (feof(server_file)) break;
		strcpy(monitor_options.server.names[++i], server_name);
	}
	if (i == MAX_SERVERS) {
		printf("*** WARNING - Exceeded number of servers ***\n");
		i--;
	}
	monitor_options.server.num = i+1;
	monitor_options.server.current = 0;
}
