/****************************************************************************
 ****************************************************************************
 **
 **   File: report.c
 **
 **   Description: Generates copy report for Xstation monitor.
 **
 ****************************************************************************
 ****************************************************************************/

/************************************************************
*                                                           *
*  Revision history:                                        *
*                                                           *
*  02/03/91  Gary Goldman, IBM Austin                       *
*                                                           *
*************************************************************/

#include "xmonitor.h"
#include <time.h>

#define WIDTH 60
#define VIEWABLE_LINES 30
/*#define FONT "rom12.500"*/
#define FONT "variable"
#define REPORT_PREFIX "report."

/***************************************************************************
 *							Locals
 ***************************************************************************/
static struct {
	Widget dialog;
	Widget scroll_bar;
	Widget label;
} report;

/***************************************************************************
 *							Functions
 ***************************************************************************/
int find_max();
char *asterick();
char *date_stamp();
char *current_date();

/***************************************************************************
 * Create dialog that displays report.
 ***************************************************************************/
void create_report()
{
    Widget scroll_w, button;
    Arg args[20];
	char dummy[WIDTH+1];
    XmString label;
	XmFontList font_list;
	XFontStruct *font;
	Dimension width, height;
    int n = 0;

	while (n < WIDTH) dummy[n++] = 'A';
	dummy[WIDTH] = '\0';

	font = XLoadQueryFont(dpy, FONT);
    font_list = XmFontListCreate(font, " ");
    report.dialog = XmCreateFormDialog(toplevel_widget,
                                    "REPORT_DIALOG", NULL, 0);

	/* get dimensions of scroll clip window  - I'm being lazy */
    label = XmStringLtoRCreate(dummy, XmSTRING_DEFAULT_CHARSET);
	width = XmStringWidth(font_list, label);
	height = XmStringHeight(font_list, label)*VIEWABLE_LINES;

	/* it seems the only way to force an initial size */
    n = 0;
    XtSetArg(args[n], XmNwidth, width); n++;
    XtSetArg(args[n], XmNheight, height); n++;
    report.dialog = XmCreateFormDialog(toplevel_widget,
                                    "REPORT_DIALOG", args, n);
    n = 0;
    XtSetArg(args[n], XmNwidth, width); n++;
    XtSetArg(args[n], XmNheight, height); n++;
    XtSetArg(args[n], XmNscrollBarDisplayPolicy, XmAS_NEEDED); n++;
    XtSetArg(args[n], XmNscrollingPolicy, XmAUTOMATIC); n++;
    XtSetArg(args[n], XmNtopAttachment, XmATTACH_FORM); n++;
    XtSetArg(args[n], XmNtopOffset, MARGIN); n++;
    XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg(args[n], XmNleftOffset, MARGIN); n++;
    XtSetArg(args[n], XmNrightOffset, MARGIN); n++;
    XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
    XtSetArg(args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
    XtSetArg(args[n], XmNbottomOffset, BOTTOM_OFFSET); n++;
    scroll_w = XmCreateScrolledWindow(report.dialog, "REPORT_STEXT", args, n);

    n = 0;
    XtSetArg(args[n], XmNlabelString, label); n++;
    XtSetArg(args[n], XmNfontList, font_list); n++;
    XtSetArg(args[n], XmNalignment, XmALIGNMENT_BEGINNING); n++;
    report.label = XmCreateLabelGadget(scroll_w, "REPORT_STEXT", args, n);
    XtManageChild(report.label);

	n = 0;
    XtSetArg(args[n], XmNminimum, 0); n++;
    XtSetArg(args[n], XmNmaximum, VIEWABLE_LINES); n++; /* temporary */
    XtSetArg(args[n], XmNincrement, 1); n++;
    XtSetArg(args[n], XmNpageIncrement, 10); n++;
    XtSetArg(args[n], XmNsliderSize, 1); n++;
    report.scroll_bar = XmCreateScrollBar(report.dialog, "REPORT_SBAR", 
											   args, n);
/*    XtManageChild(report.scroll_bar);*/

	XmScrolledWindowSetAreas(scroll_w, NULL, report.scroll_bar, report.label);
    XtManageChild(scroll_w);

    n = 0;
    label = XmStringLtoRCreate("Close", XmSTRING_DEFAULT_CHARSET);
    XtSetArg(args[n], XmNshowAsDefault, 1); n++;
    XtSetArg(args[n], XmNlabelString, label); n++;
    XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg(args[n], XmNleftOffset, MARGIN); n++;
    XtSetArg(args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
    XtSetArg(args[n], XmNbottomOffset, MARGIN); n++;
    button = XmCreatePushButton(report.dialog, "REPORT_CLOSE", args, n);
    XtManageChild(button);

    n = 0;
    XtSetArg(args[n], XmNdefaultButton, button); n++;
    XtSetValues(report.dialog, args, n);

    XmStringFree(label);
}

/***************************************************************************
 * Generates report on stats buffered.
 ***************************************************************************/
void gen_report()
{
	FILE *rpt_file;
	int group_num;

	monitor_options.rpt_name[0] = '\0';
	strcpy(monitor_options.rpt_name, REPORT_PREFIX);
	strcat(monitor_options.rpt_name, date_stamp());
   	if ((rpt_file = fopen(monitor_options.rpt_name, "w")) == NULL) {
   		printf("Cannot open %s file\n", monitor_options.rpt_name);
		return;
	}
	fprintf(rpt_file, "%s\n", current_date());

	FOR_ALL_GROUPS(group_num) group[group_num].rpt_method(rpt_file);

    fclose(rpt_file);
}

/***************************************************************************
 * Returns a date stamp.
 ***************************************************************************/
char *date_stamp()
{
	time_t ltime;
	struct tm *local;
	char string[11], *format = "%m%d%H%M%S";
	int length = 10;

	ltime = time(NULL);
	local = localtime(&ltime);
	NLstrtime(string, length, format, local);
	return(string);
}

/***************************************************************************
 * Returns current date.
 ***************************************************************************/
char *current_date()
{
	time_t ltime;
	struct tm *local;
	char string[28], *format = "%lh %d, %Y %r";
	int length = 27;

	ltime = time(NULL);
	local = localtime(&ltime);
	NLstrtime(string, length, format, local);
	return(string);
}

/***************************************************************************
 * Generates report on stats buffered.
 ***************************************************************************/
int find_max(values, size)
	int *values, size;
{
	int i, max_value = 0;

	for (i = 0; i < size; i++)
		if (values[i] > max_value) max_value = values[i];
	return(max_value);
}
/***************************************************************************
 * Prefixes value with asterick and passes back string.
 *** ************************************************************************/
char *asterick(value)
	int value;
{
	char *string;
	int i;

	string = (char *) malloc(sizeof(char)*(10));
	sprintf(string, "%9d\0", value);
	for (i  = 0; i < 9; i++)
		if (string[i] != ' ') {
			string[i-1] = '*';
			return(string);
		}
	printf("*** error in asterick function ***\n");
	return(string);
}

/***************************************************************************
 * Posts report popup menu.
 ***************************************************************************/
void post_report(w, popup, event)
	Widget w;
	Widget popup;
	XButtonEvent *event;
{
	if (event->button == Button3){
		XmMenuPosition(popup, event);
		XtManageChild(popup);
	}
}

/***************************************************************************
 * Displays last report in scrolled text widget.
 ***************************************************************************/
void display_report()
{
	char *buff;
	Arg arg;
	XmString label;
	int line_count;

	/* put report name in dialog title */
    label = XmStringLtoRCreate(monitor_options.rpt_name, 
			XmSTRING_DEFAULT_CHARSET);
    XtSetArg(arg, XmNdialogTitle, label); 
	XtSetValues(report.dialog, &arg, 1);

	/* load text into label */
	buff = read_file(monitor_options.rpt_name);
	label = XmStringLtoRCreate(buff, XmSTRING_DEFAULT_CHARSET);
	XtSetArg(arg, XmNlabelString, label);
	XtSetValues(report.label, &arg, 1);

	/* get the # lines in the string to change vertical scroll bar parms */
	line_count = XmStringLineCount(label);
	XtSetArg(arg, XmNmaximum, 
			 (line_count > VIEWABLE_LINES) ? line_count : VIEWABLE_LINES);
	XtSetValues(report.scroll_bar, &arg, 1);

	XtManageChild(report.dialog);
	XmStringFree(label);
}
