/****************************************************************************
 ****************************************************************************
 **
 **   File: monitor.c
 **
 **   Description: Xstation monitor.
 **
 ****************************************************************************
 ****************************************************************************/

/************************************************************
*                                                           *
*  Revision history:                                        *
*                                                           *
*  02/03/91  Gary Goldman, IBM Austin                       *
*                                                           *
*************************************************************/

#define MAIN
#include "xmonitor.h"
#include "xmonitor.bit"

/*****************************************************************************
 *                        Globals
 *****************************************************************************/
Bool print = FALSE;  	/* print out values and coordinates */
int sample_count;

/*****************************************************************************
 *                        Locals
 *****************************************************************************/
static Bool synch = FALSE;  /* put in synchronis mode for debug */
static char *font_name[] = {"Bld14.500", "rom14.500", "fixed", "variable", 
						    "fixed", "Erg14.500"};
static XFontStruct *fonts[NUM_FONTS]; 

/*****************************************************************************
 *          		Function Declarations
 *****************************************************************************/
static void get_display();
static void set_appl_widget();
static void get_fonts();
static void create_colors();
extern void read_template();
int  conv_x_coord();
int  conv_y_coord();
void init_XSSTAT();
void draw_axes();
void shift_graph();
static Bool get_viewport();
void draw_graph();
void erase_line();
void set_counter();

/***************************************************************************
 * main - initialize then let XtMainLoop take over.
 ***************************************************************************/
main(argc, argv)
    unsigned int argc;
    char *argv[];    
{
	int i;

	/* check for command line arguments */
	for (i = 1; i < argc; i++) {
		if (!strcmp(argv[i],"-s"))
			synch = TRUE;
		if (!strcmp(argv[i],"-print"))
			print = TRUE;
	}

    /* Initialize the X Toolkit and get back a top level shell widget. */
    toplevel_widget = XtInitialize(argv[0], "Xstation_monitor", NULL, 0, 
								   &argc, argv);
	get_display();
	create_colors();
	set_appl_widget();
	get_fonts();
	read_template("default");
	create_monitor();

    /* Sit around forever waiting to process X-events.  We never leave
       XtMainLoop. From here on only callback routines are executed. */
    XtMainLoop();
}

/***************************************************************************
 * Get basic display and screen parameters.
 ***************************************************************************/
static void get_display()
{
    dpy = XtDisplay(toplevel_widget);
	toplevel_context = XtWidgetToApplicationContext(toplevel_widget);
    if (synch) XSynchronize(dpy, 1);
    screen = XtScreen(toplevel_widget);
	gc_general = DefaultGCOfScreen(screen);
}

/***************************************************************************
 * Customize the application shell widget (toplevel).
 ***************************************************************************/
static void set_appl_widget()
{
    Window Icon_Win;
    Pixmap Icon_Pix;
    Pixel backg, foreg, border;
    int border_width = 1;
    XSetWindowAttributes attributes;
    Arg widget_args[10];
	int n = 0;

    XtSetArg(widget_args[n], XmNtitle, "Xstation Monitor"); n++;

    /* Icon Stuff */
	foreg = black;
	backg = white;
    border = black;
    Icon_Pix = XCreatePixmapFromBitmapData(dpy, DefaultRootWindow(dpy),
            xmonitor_bits, xmonitor_width, xmonitor_height, foreg, backg,
            DefaultDepthOfScreen(screen));
    attributes.background_pixmap = Icon_Pix;
    attributes.border_pixel = border;
    Icon_Win = XCreateWindow(dpy, DefaultRootWindow(dpy),
                0, 0, xmonitor_width, xmonitor_height, border_width,
                DefaultDepthOfScreen(screen), InputOutput,
                DefaultVisualOfScreen(screen), CWBackPixmap | CWBorderPixel,
                &attributes);
    XtSetArg(widget_args[n], XmNiconPixmap, Icon_Pix);  n++;
    XtSetArg(widget_args[n], XmNiconWindow, Icon_Win); n++;
    XtSetArg(widget_args[n], XmNiconName, "Xstation"); n++;
    XtSetValues(toplevel_widget, widget_args, n);
}

/***************************************************************************
 * Create font lists to be used for latter widget creations/modifications.
 ***************************************************************************/
static void get_fonts()
{
	int n = 0;

    fonts[n] = XLoadQueryFont(dpy, font_name[n]);
    large_font = XmFontListCreate(fonts[n], " "); n++;
    fonts[n] = XLoadQueryFont(dpy, font_name[n]);
    label_font = XmFontListCreate(fonts[n], " "); n++;
    fonts[n] = XLoadQueryFont(dpy, font_name[n]);
    sublabel_font = XmFontListCreate (fonts[n], " "); n++;
    fonts[n] = XLoadQueryFont(dpy, font_name[n]);
    widget_font = XmFontListCreate(fonts[n], " "); n++;
    fonts[n] = XLoadQueryFont(dpy, font_name[n]);
    tiny_font = XmFontListCreate(fonts[n], " "); n++;
    fonts[n] = XLoadQueryFont(dpy, font_name[n]);
    text_font = XmFontListCreate(fonts[n], " "); n++;
}

/***************************************************************************
 * Initialize color stuff.
 ***************************************************************************/
static void create_colors()
{
    cmap = DefaultColormapOfScreen(screen);
    black = BlackPixelOfScreen(screen);
    white = WhitePixelOfScreen(screen);
}

/***************************************************************************
 * Initialize the server statistics library.
 ***************************************************************************/
void init_XSSTAT()
{
	int event_basep, error_basep, current;

	current = monitor_options.server.current;
	monitor_options.server.dpy = 
		XOpenDisplay(
	 		monitor_options.server.names[monitor_options.server.current]);
	if (!XSSTATQueryExtension(monitor_options.server.dpy, 
		&event_basep, &error_basep)) {
		fprintf(stderr, "SSTAT extension not installed\n", stderr);
	}
	else 
		XSSTATSetStatsOn(monitor_options.server.dpy, 
						 monitor_options.history_size);
}

/***************************************************************************
 * Draws graph axes on Form window.
 ***************************************************************************/
void draw_axes(graph_num)
	int graph_num;
{
	XPoint point[4];
	int line_width = 0, x_tick, y_tick, i;

	if (foreground_general != black) XSetForeground(dpy, gc_general, black);
	XSetLineAttributes(dpy, gc_general, line_width, LineSolid, CapButt, 
					   JoinMiter);
	if (!line_width) line_width = 1;
	point[0].x = point[1].x = graph_prop.left_offset-line_width;
	point[2].x = point[3].x = graph_prop.left_offset+
			(graph[graph_num].view.size.x*graph[graph_num].geom.inc_pixels.x);
	point[0].y = point[3].y = graph_prop.widget_offset;
	point[1].y = point[2].y = point[0].y+
			(graph[graph_num].view.size.y*graph[graph_num].geom.inc_pixels.y)+
			line_width;
	XDrawLines(dpy, XtWindow(graph[graph_num].form), gc_general, point, 4,
				CoordModeOrigin);

	/* draw tick marks */
	x_tick = point[0].x-graph_prop.tick_size;
	for (i = 0; i <= graph[graph_num].view.size.y; i++) {
		y_tick = point[0].y+(i*graph[graph_num].geom.inc_pixels.y);
		XDrawLine(dpy, XtWindow(graph[graph_num].form), gc_general, 
				x_tick, y_tick, point[0].x, y_tick);
	}
	y_tick = point[1].y+graph_prop.tick_size;
	for (i = 0; i <= graph[graph_num].view.size.x; i++) {
		x_tick = i*(graph[graph_num].geom.inc_pixels.x)+point[0].x;
		XDrawLine(dpy, XtWindow(graph[graph_num].form), gc_general, 
				  x_tick, y_tick, x_tick, point[1].y);
	}
	x_tick = point[2].x+graph_prop.tick_size;
	for (i = 0; i <= graph[graph_num].view.size.y; i++) {
		y_tick = point[0].y+(i*graph[graph_num].geom.inc_pixels.y);
		XDrawLine(dpy, XtWindow(graph[graph_num].form), gc_general, 
				  x_tick, y_tick, point[2].x, y_tick);
	}
}

/***************************************************************************
 * Converts x global value to x viewport coord.
 ***************************************************************************/
int conv_x_coord(x, graph_num)
	int x, graph_num;
{
	int coord;

	coord = 
		(x-graph[graph_num].view.start.x)*graph[graph_num].geom.inc_pixels.x;
	return(coord);
}
/***************************************************************************
 * Converts y global value to y viewport coord.
 ***************************************************************************/
int conv_y_coord(data, graph_num)
	int data, graph_num;
{
	int coord;

	coord = (graph[graph_num].view.end.y - 
			((float)data/graph[graph_num].geom.inc_units.y))
			*graph[graph_num].geom.inc_pixels.y;
	return(coord);
}

/***************************************************************************
 * Shift graph data one increment left.
 ***************************************************************************/
void shift_graph(graph_num)
	int graph_num;
{
	LINE *line_ptr;
	int line_idx, j;

	draw_graph(graph_num, TRUE);
	FOR_LINES_IN_GRAPH(line_idx, graph_num) {
		line_ptr = graph[graph_num].line[line_idx];
		if (line_ptr->start_of_data) line_ptr->start_of_data--;
		for (j = line_ptr->start_of_data; j < graph[graph_num].view.current; 
				j++) {
			line_ptr->value[j] = line_ptr->value[j+1];
			line_ptr->point[j].y = line_ptr->point[j+1].y;
		}
	}
	graph[graph_num].view.current--;
}

/***************************************************************************
 * Returns start and end of visible data (lines).
 ***************************************************************************/
static Bool get_viewport(line_ptr, start, end)
	LINE *line_ptr;
	int *start, *end;
{
	GRAPH *graph_ptr;

	graph_ptr = line_ptr->graph;
	*start = (line_ptr->start_of_data > graph_ptr->view.start.x)
			? line_ptr->start_of_data : graph_ptr->view.start.x;
	if (graph_ptr->view.current >= graph_ptr->view.start.x &&
		graph_ptr->view.current <= graph_ptr->view.end.x)
		*end = graph_ptr->view.current;
	else
		*end = graph_ptr->view.end.x;
	/* establish if any visible lines */
	return((*start >= *end) ? FALSE : TRUE);
}

/***************************************************************************
 * Draw or erases graphs within the present viewport.
 ***************************************************************************/
void draw_graph(gr_num, erase)
	int gr_num;
	Bool erase;
{
	LINE *line_ptr;
	int i, num_pnts, start, end;
	GC gc_used;

	for (i = 0; i < graph[gr_num].num_lines; i++) {
		line_ptr = graph[gr_num].line[i];
		if (line_ptr->monitor) {
			if (get_viewport(line_ptr, &start, &end)) {
				gc_used = (erase) ? graph[gr_num].gc : line_ptr->gc;
				num_pnts = end-start+1;
				XDrawLines(dpy, XtWindow(graph[gr_num].d_area), gc_used, 
					  &line_ptr->point[start], num_pnts, CoordModeOrigin);	
			}
		}
	}
}

/***************************************************************************
 * Erases a single line in the case of toggle off.
 ***************************************************************************/
void erase_line(line_ptr)
	LINE *line_ptr;
{
	GRAPH *graph_ptr;
	int num_pnts, start, end;

	if (get_viewport(line_ptr, &start, &end)) {
		graph_ptr = line_ptr->graph;
		num_pnts = end-start+1;
		XDrawLines(dpy, XtWindow(graph_ptr->d_area), graph_ptr->gc, 
				line_ptr->point[start], num_pnts, CoordModeOrigin);	
	}
}

/***************************************************************************
 * Sets sample count label.
 ***************************************************************************/
void set_counter(count)
	int count;
{
	char digits[33];
	Arg label_arg;
	XmString label;

    sprintf(digits, "%d\0", count);
    label = XmStringLtoRCreate(digits, XmSTRING_DEFAULT_CHARSET);
    XtSetArg(label_arg, XmNlabelString, label);
    XtSetValues(sample_count_label, &label_arg, 1);
	XmStringFree(label);
}

/***************************************************************************
 * Reads the contents of an entire file and returns as it's contents.
 ***************************************************************************/
char *read_file(fileptr)
	char *fileptr;
{
    static char *retbuff;
    int i, fd, flen;

    if ((fd = open(fileptr, 0)) < 0) {
        printf("Cannot open file %s\n", fileptr);
        return((char *) NULL);
    }
    flen = lseek(fd, 0, 2);
    lseek(fd, 0, 0);
    retbuff = (char*) calloc(1, flen + 1);
    if (read (fd, retbuff, flen) <= 0) {
        printf("Error reading file %s\n", fileptr);
        return((char *) NULL);
    }
    close(fd);

    i = strlen(retbuff)-1;
    if (retbuff[i] == '\n') retbuff[i] = '\0';

    return(retbuff);
}
