/****************************************************************************
 ****************************************************************************
 **
 **   File: xmonitor.h
 **
 **   Description: header file for xmonitor.c
 **
 ****************************************************************************
 ****************************************************************************/

/************************************************************
*                                                           *
*  Revision history:                                        *
*                                                           *
*  10/09/90  Gary Goldman, IBM Austin   Initial Version     *
*                                                           *
*************************************************************/

#ifndef MAIN
#define EXTERN extern
#else
#define EXTERN
#endif

#include <stdio.h>
#include <math.h>
#include <Xm/Xm.h>
#include <X11/MwmUtil.h>

/*****************************************************************************
 *                      Application Defines
 *****************************************************************************/
#define MAX_GROUPS 2
#define RESOURCE_GROUP 0
#define XPROTOCOL_GROUP 1
#define NUM_GRAPHS 4
#define NUM_RESOURCES 7
#define NUM_PROTOCOLS 120
#define NUM_LINES NUM_RESOURCES+NUM_PROTOCOLS
#define MAX_SERVERS 50      /* max number of servers in the server list */
#define NUM_FONTS 6           /* number of fonts used */
#define ON_DEMAND 0
#define TIMED_INTERVAL 1
#define HORIZONTAL 0
#define VERTICAL 1
#define MARGIN 10
#define BOTTOM_OFFSET 2*MARGIN+30 /* placement above standard bottom dialog */
								  /*   buttons - enables resizing */

#define FOR_ALL_GROUPS(group_num) \
	for (group_num = 0; group_num < MAX_GROUPS; group_num++)
#define FOR_GRAPHS_IN_GROUP(graph_idx, group_num) \
	for (graph_idx = 0; graph_idx < group[group_num].num_graphs; graph_idx++)
#define FOR_LINES_IN_GRAPH(line_idx, graph_num) \
	for (line_idx = 0; line_idx < graph[graph_num].num_lines; line_idx++)
#define FOR_LINES_IN_GRAPH_PTR(line_idx, graph_ptr) \
	for (line_idx = 0; line_idx < graph_ptr->num_lines; line_idx++)
#define FOR_ALL_GRAPHS(graph_num) \
	for (graph_num = 0; graph_num < NUM_GRAPHS; graph_num++)
#define FOR_ALL_LINES(line_num) \
	for (line_num = 0; line_num < NUM_LINES; line_num++)
#define LOOKUP_GRAPH(group_num, grph_idx, grph_num) \
	FOR_ALL_GROUPS(group_num) \
	FOR_GRAPHS_IN_GROUP(graph_idx, group_num) \
	FOR_ALL_GRAPHS(graph_num)

/*****************************************************************************
 *                              General
 *****************************************************************************/

EXTERN Display *dpy;
EXTERN Widget toplevel_widget;          /* application widget id */
EXTERN XtAppContext toplevel_context;
EXTERN Screen *screen;
EXTERN GC gc_general;
EXTERN Pixel foreground_general = 9999;
EXTERN XmFontList large_font, label_font, sublabel_font, widget_font, 
				  tiny_font, text_font;

/*****************************************************************************
 *              				Color
 *****************************************************************************/

EXTERN Colormap cmap;
EXTERN Pixel black, white,  green, blue, red, pink, thistle, turquoise, 
			 skyblue, grey, forestgreen; 

/*****************************************************************************
 *              					Graphs
 *****************************************************************************/
typedef struct {
	int x;
	int y;
}XYdata;

EXTERN struct _GEOM {
	XYdata position;		/* initial position */
	XYdata inc_pixels;		/* pixels per increment */
	XYdata inc_units;     	/* units per increment */
	XYdata max_inc;    		/* max number of increments */
};							/*    which is also max amount of data */

EXTERN struct _VIEW {
	int current;	/* index to current position into line.data */
	XYdata size; 	/* width & height of viewport in inc (end-start) */
	XYdata start;	/* start of viewport _| (x=index into line.data) */
	XYdata end;		/* end of viewport    | (y=viewport min & max in inc) */
};

typedef struct _LINE {
	struct _GRAPH *graph;	/* pointer to parent graph */
	Bool monitor;			/* on or off for monitoring */
	Pixel color;
	int width;
	GC gc;
	int *value;			/* size this to graph[gr_num].geom.max_inc.x */
	XPoint *point;		
	int start_of_data; 	/* left most position of data (viewport independent) */
	char name[31];
	int id;
} LINE;
LINE line[NUM_LINES];

EXTERN struct _LABELS {
	int num;		/* number of visible labels */
	int inc;		/* number of increments between labels */
	int *value;		/* current values of labels */
	Widget *widget;	/* label widgets */
};

typedef struct _GRAPH {
	struct _GROUP *group; 		/* pointer to parent group */
	Widget dialog, form, d_area;
	Pixel background;   		/* background color of graphs */
	int line_width;				/* widest line within the graph */
	GC gc;						/* gc takes care of erase */
	char *title;
	Bool start;
	struct _GEOM geom;  		/* controls overall geometry of graphs */
	struct _VIEW view;   		/* viewport established in graph */
	LINE **line; 				/* pointers to lines in graph */
	int num_lines;  			/* number of lines in the graph */
	struct _LABELS labels[2]; 	/* y axis labels (left/right) */
} GRAPH; 
GRAPH graph[NUM_GRAPHS];

typedef struct _GROUP {
	GRAPH **graph;			/* pointers to graphs in group */
	int num_graphs; 		/* number of graphs belonging to this group */
	char *title;
	void (*value_method)();	/* function that gets group values */
	void (*rpt_method)();   /* function that generates report */
} GROUP;
GROUP group[MAX_GROUPS];

EXTERN struct {
	int left_offset;	/* general offset of graph from other widgets */
	int widget_offset;  /* offsets from other widgets */
	int tick_size;
} graph_prop = {40,15,5};

/*****************************************************************************
 *              		Monitor Option Globals
 *****************************************************************************/
EXTERN struct _SERVER {
	char names[MAX_SERVERS][30]; /* names out of server.list file */
	int num;
	int current;  /* index to active server in server_names */
	Display *dpy; /* server display id that will be monitored */
};

EXTERN struct {
	struct _SERVER server;
	int sample_rate;
	int sample_time[2];
	int history_size;
	Bool client_filter;
	Bool slock;
	char file_name[30];
	FILE *file;
	char rpt_name[30];
} monitor_options;
EXTERN Widget sample_count_label, sample_rate_dialog, 
			  sample_rate_scale[2], client_filter_widget[3], 
			  client_filter_dialog;

/*****************************************************************************
 *              				Misc Stuff	
 *****************************************************************************/
EXTERN Widget monitor_sample_dialog, monitor_in_progress_dialog;
EXTERN Widget protocols_dialog;
Dimension protocol_width = 0;

/*****************************************************************************
 *              					Help
 *****************************************************************************/
EXTERN struct {
	Widget dialog;
	char title[30];
} help[50];

/*****************************************************************************
*          				Function Declarations
*****************************************************************************/
EXTERN void create_monitor();
EXTERN char *read_file();
EXTERN void get_resources();
EXTERN void get_protocols();
EXTERN void rpt_resources();
EXTERN void rpt_protocols();

/*****************************************************************************
*          Callback Function Declarations
*****************************************************************************/
EXTERN void exit_proc();

EXTERN void expose_graph();
EXTERN void expose_form();
EXTERN void scroll_graph();
EXTERN void display_graph();
EXTERN void remove_graph();

EXTERN void activate_monitor();
EXTERN void stop_monitor();
EXTERN void set_line_monitor();
EXTERN XtTimerCallbackProc sample_stats();
EXTERN void set_server();
EXTERN void set_sample_rate();
EXTERN void set_sample_interval();
EXTERN void set_history_size();
EXTERN void set_client_filter();
EXTERN void set_scroll_lock();
EXTERN void activate_protocols();
EXTERN void close_protocols();
EXTERN void gen_report();
EXTERN void display_report();
EXTERN void post_report();

EXTERN void activate_help();

/*****************************************************************************
*		         Widget Creation Functions 
*****************************************************************************/

EXTERN void create_mainwindow();
EXTERN void create_graph_labels();
EXTERN void set_graph_labels();
